@@ -1,4 +1,3 @@
-import errno
 import json
 import os
 import socket
@@ -34,7 +33,8 @@ class TimeoutTimer:
 
         :param timeout: time out interval [s] or None (no timeout)
         :param sleep: sleep interval [s] (>= 0: do sleep call, <0: don't call sleep)
-                      or None (autocompute: use 10% of timeout, or 1s for no timeout)
+                      or None (autocompute: use 10% of timeout [but not more than 60s],
+                      or 1s for no timeout)
         """
         if timeout is not None and timeout < 0:
             raise ValueError("timeout must be >= 0")
@@ -43,7 +43,7 @@ class TimeoutTimer:
             if timeout is None:
                 sleep = 1.0
             else:
-                sleep = timeout / 10.0
+                sleep = min(60.0, timeout / 10.0)
         self.sleep_interval = sleep
         self.start_time = None
         self.end_time = None
@@ -101,9 +101,11 @@ class NotMyLock(LockErrorT):
 class ExclusiveLock:
     """An exclusive Lock based on mkdir fs operation being atomic.
 
-    If possible, try to use the contextmanager here like:
-    with ExclusiveLock(...) as lock:
-        ...
+    If possible, try to use the contextmanager here like::
+
+        with ExclusiveLock(...) as lock:
+            ...
+
     This makes sure the lock is released again if the block is left, no
     matter how (e.g. if an exception occurred).
     """
@@ -112,7 +114,7 @@ class ExclusiveLock:
         self.sleep = sleep
         self.path = os.path.abspath(path)
         self.id = id or get_id()
-        self.unique_name  = os.path.join(self.path, "%s.%d-%x" % self.id)
+        self.unique_name = os.path.join(self.path, "%s.%d-%x" % self.id)
 
     def __enter__(self):
         return self.acquire()
@@ -132,14 +134,13 @@ class ExclusiveLock:
         while True:
             try:
                 os.mkdir(self.path)
+            except FileExistsError:  # already locked
+                if self.by_me():
+                    return self
+                if timer.timed_out_or_sleep():
+                    raise LockTimeout(self.path)
             except OSError as err:
-                if err.errno == errno.EEXIST:  # already locked
-                    if self.by_me():
-                        return self
-                    if timer.timed_out_or_sleep():
-                        raise LockTimeout(self.path)
-                else:
-                    raise LockFailed(self.path, str(err))
+                raise LockFailed(self.path, str(err)) from None
             else:
                 with open(self.unique_name, "wb"):
                     pass
@@ -181,12 +182,8 @@ class LockRoster:
         try:
             with open(self.path) as f:
                 data = json.load(f)
-        except IOError as err:
-            if err.errno != errno.ENOENT:
-                raise
-            data = {}
-        except ValueError:
-            # corrupt/empty roster file?
+        except (FileNotFoundError, ValueError):
+            # no or corrupt/empty roster file?
             data = {}
         return data
 
@@ -197,9 +194,8 @@ class LockRoster:
     def remove(self):
         try:
             os.unlink(self.path)
-        except OSError as e:
-            if e.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
 
     def get(self, key):
         roster = self.load()
@@ -228,9 +224,11 @@ class UpgradableLock:
     noone is allowed reading) and read access to a resource needs a shared
     lock (multiple readers are allowed).
 
-    If possible, try to use the contextmanager here like:
-    with UpgradableLock(...) as lock:
-        ...
+    If possible, try to use the contextmanager here like::
+
+        with UpgradableLock(...) as lock:
+            ...
+
     This makes sure the lock is released again if the block is left, no
     matter how (e.g. if an exception occurred).
     """
@@ -278,9 +276,11 @@ class UpgradableLock:
             try:
                 if remove is not None:
                     self._roster.modify(remove, REMOVE)
-                    remove = None
                 if len(self._roster.get(SHARED)) == 0:
                     return  # we are the only one and we keep the lock!
+                # restore the roster state as before (undo the roster change):
+                if remove is not None:
+                    self._roster.modify(remove, ADD)
             except:
                 # avoid orphan lock when an exception happens here, e.g. Ctrl-C!
                 self._lock.release()
